@@ -602,32 +602,21 @@ EditFormButtonsCommand::EditFormButtonsCommand(Okular::DocumentPrivate *docPriv,
     setText(i18nc("Edit the state of a group of form buttons", "edit form button states"));
     for (const FormFieldButton *formButton : std::as_const(m_formButtons)) {
         m_prevButtonStates.append(formButton->state());
+        m_pageNumbers.append(formButton->page()->number());
     }
 }
 
 void EditFormButtonsCommand::undo()
 {
     clearFormButtonStates();
+    QSet<int> extraPages;
     for (int i = 0; i < m_formButtons.size(); i++) {
         bool checked = m_prevButtonStates.at(i);
         if (checked) {
             m_formButtons.at(i)->setState(checked);
         }
-    }
-
-    Okular::NormalizedRect boundingRect = buildBoundingRectangleForButtons(m_formButtons);
-    moveViewportIfBoundingRectNotFullyVisible(boundingRect, m_docPriv, m_pageNumber);
-    Q_EMIT m_docPriv->m_parent->formButtonsChangedByUndoRedo(m_pageNumber, m_formButtons);
-    m_docPriv->notifyFormChanges(m_pageNumber);
-}
-
-void EditFormButtonsCommand::redo()
-{
-    clearFormButtonStates();
-    for (int i = 0; i < m_formButtons.size(); i++) {
-        bool checked = m_newButtonStates.at(i);
-        if (checked) {
-            m_formButtons.at(i)->setState(checked);
+        if (m_pageNumbers.at(i) != m_pageNumber) {
+            extraPages << m_pageNumbers.at(i);
         }
     }
 
@@ -635,14 +624,40 @@ void EditFormButtonsCommand::redo()
     moveViewportIfBoundingRectNotFullyVisible(boundingRect, m_docPriv, m_pageNumber);
     Q_EMIT m_docPriv->m_parent->formButtonsChangedByUndoRedo(m_pageNumber, m_formButtons);
     m_docPriv->notifyFormChanges(m_pageNumber);
+    for (auto page : std::as_const(extraPages)) {
+        m_docPriv->notifyFormChanges(page);
+    }
+}
+
+void EditFormButtonsCommand::redo()
+{
+    clearFormButtonStates();
+    QSet<int> extraPages;
+    for (int i = 0; i < m_formButtons.size(); i++) {
+        bool checked = m_newButtonStates.at(i);
+        if (checked) {
+            m_formButtons.at(i)->setState(checked);
+        }
+        if (m_pageNumbers.at(i) != m_pageNumber) {
+            extraPages << m_pageNumbers.at(i);
+        }
+    }
+
+    Okular::NormalizedRect boundingRect = buildBoundingRectangleForButtons(m_formButtons);
+    moveViewportIfBoundingRectNotFullyVisible(boundingRect, m_docPriv, m_pageNumber);
+    Q_EMIT m_docPriv->m_parent->formButtonsChangedByUndoRedo(m_pageNumber, m_formButtons);
+    m_docPriv->notifyFormChanges(m_pageNumber);
+    for (auto page : std::as_const(extraPages)) {
+        m_docPriv->notifyFormChanges(page);
+    }
 }
 
 bool EditFormButtonsCommand::refreshInternalPageReferences(const QVector<Okular::Page *> &newPagesVector)
 {
     const QList<FormFieldButton *> oldFormButtons = m_formButtons;
     m_formButtons.clear();
-    for (FormFieldButton *oldFormButton : oldFormButtons) {
-        FormFieldButton *button = dynamic_cast<FormFieldButton *>(Okular::PagePrivate::findEquivalentForm(newPagesVector[m_pageNumber], oldFormButton));
+    for (int i = 0; i < oldFormButtons.size(); i++) {
+        FormFieldButton *button = dynamic_cast<FormFieldButton *>(Okular::PagePrivate::findEquivalentForm(newPagesVector[m_pageNumbers[i]], oldFormButtons[i]));
         if (!button) {
             return false;
         }
