@@ -49,10 +49,11 @@ fontPool::fontPool(bool useFontHinting)
 #ifdef HAVE_FREETYPE
     // Initialize the Freetype Library
     if (FT_Init_FreeType(&FreeType_library) != 0) {
-        qCCritical(OkularDviDebug) << "Cannot load the FreeType library. KDVI proceeds without FreeType support." << endl;
+        qCCritical(OkularDviDebug) << "Cannot load the FreeType library. KDVI proceeds without FreeType support.";
         FreeType_could_be_loaded = false;
-    } else
+    } else {
         FreeType_could_be_loaded = true;
+    }
 #endif
 
     // Check if the QT library supports the alpha channel of
@@ -96,8 +97,9 @@ fontPool::~fontPool()
     fontList.clear();
 
 #ifdef HAVE_FREETYPE
-    if (FreeType_could_be_loaded == true)
+    if (FreeType_could_be_loaded == true) {
         FT_Done_FreeType(FreeType_library);
+    }
 #endif
 }
 
@@ -161,8 +163,9 @@ bool fontPool::areFontsLocated()
     QList<TeXFontDefinition *>::const_iterator cit_fontp = fontList.constBegin();
     for (; cit_fontp != fontList.constEnd(); ++cit_fontp) {
         TeXFontDefinition *fontp = *cit_fontp;
-        if (!fontp->isLocated())
+        if (!fontp->isLocated()) {
             return false;
+        }
     }
 
 #ifdef DEBUG_FONTPOOL
@@ -187,28 +190,30 @@ void fontPool::locateFonts()
 
     // If still not all fonts are found, look again, this time with
     // on-demand generation of PK fonts enabled.
-    if (!areFontsLocated())
+    if (!areFontsLocated()) {
         locateFonts(true, false);
+    }
 
     // If still not all fonts are found, we look for TFM files as a last
     // resort, so that we can at least draw filled rectangles for
     // characters.
-    if (!areFontsLocated())
+    if (!areFontsLocated()) {
         locateFonts(false, true);
+    }
 
     // If still not all fonts are found, we give up. We mark all fonts
     // as 'located', so that we won't look for them any more, and
     // present an error message to the user.
     if (!areFontsLocated()) {
         markFontsAsLocated();
-        emit error(i18n("<qt><p>Okular was not able to locate all the font files "
-                        "which are necessary to display the current DVI file. "
-                        "Your document might be unreadable.</p>"
-                        "<p><small><b>PATH:</b> %1</small></p>"
-                        "<p><small>%2</small></p></qt>",
-                        QString::fromLocal8Bit(qgetenv("PATH")),
-                        kpsewhichOutput.replace(QLatin1String("\n"), QLatin1String("<br/>"))),
-                   -1);
+        Q_EMIT error(i18n("<qt><p>Okular was not able to locate all the font files "
+                          "which are necessary to display the current DVI file. "
+                          "Your document might be unreadable.</p>"
+                          "<p><small><b>PATH:</b> %1</small></p>"
+                          "<p><small>%2</small></p></qt>",
+                          QString::fromLocal8Bit(qgetenv("PATH")),
+                          kpsewhichOutput.replace(QLatin1String("\n"), QLatin1String("<br/>"))),
+                     -1);
     }
 }
 
@@ -241,14 +246,15 @@ void fontPool::locateFonts(bool makePK, bool locateTFMonly, bool *virtualFontsFo
         if (!fontp->isLocated()) {
             numFontsInJob++;
 
-            if (locateTFMonly == true)
+            if (locateTFMonly == true) {
                 kpsewhich_args << QStringLiteral("%1.tfm").arg(fontp->fontname);
-            else {
+            } else {
 #ifdef HAVE_FREETYPE
                 if (FreeType_could_be_loaded == true) {
                     const QString &filename = fontsByTeXName.findFileName(fontp->fontname);
-                    if (!filename.isEmpty())
+                    if (!filename.isEmpty()) {
                         kpsewhich_args << QStringLiteral("%1").arg(filename);
+                    }
                 }
 #endif
                 kpsewhich_args << QStringLiteral("%1.vf").arg(fontp->fontname) << QStringLiteral("%1.1200pk").arg(fontp->fontname);
@@ -256,14 +262,15 @@ void fontPool::locateFonts(bool makePK, bool locateTFMonly, bool *virtualFontsFo
         }
     }
 
-    if (numFontsInJob == 0)
+    if (numFontsInJob == 0) {
         return;
+    }
 
     // If PK fonts are generated, the kpsewhich command will re-route
     // the output of MetaFont into its stderr. Here we make sure this
     // output is intercepted and parsed.
-    kpsewhich_ = new QProcess();
-    connect(kpsewhich_, &QProcess::readyReadStandardError, this, &fontPool::mf_output_receiver);
+    kpsewhich_ = std::make_unique<QProcess>();
+    connect(kpsewhich_.get(), &QProcess::readyReadStandardError, this, &fontPool::mf_output_receiver);
 
     // Now run... kpsewhich. In case of error, kick up a fuss.
     // This string is not going to be quoted, as it might be were it
@@ -274,19 +281,19 @@ void fontPool::locateFonts(bool makePK, bool locateTFMonly, bool *virtualFontsFo
     kpsewhich_->start(kpsewhichFullPath, kpsewhich_args, QIODevice::ReadOnly | QIODevice::Text);
     if (!kpsewhich_->waitForStarted()) {
         QApplication::restoreOverrideCursor();
-        emit error(i18n("<qt><p>There were problems running <em>kpsewhich</em>. As a result, "
-                        "some font files could not be located, and your document might be unreadable.<br/>"
-                        "Possible reason: the <em>kpsewhich</em> program is perhaps not installed on your system, "
-                        "or it cannot be found in the current search path.</p>"
-                        "<p><small><b>PATH:</b> %1</small></p>"
-                        "<p><small>%2</small></p></qt>",
-                        QString::fromLocal8Bit(qgetenv("PATH")),
-                        kpsewhichOutput.replace(QLatin1String("\n"), QLatin1String("<br/>"))),
-                   -1);
+        Q_EMIT error(i18n("<qt><p>There were problems running <em>kpsewhich</em>. As a result, "
+                          "some font files could not be located, and your document might be unreadable.<br/>"
+                          "Possible reason: the <em>kpsewhich</em> program is perhaps not installed on your system, "
+                          "or it cannot be found in the current search path.</p>"
+                          "<p><small><b>PATH:</b> %1</small></p>"
+                          "<p><small>%2</small></p></qt>",
+                          QString::fromLocal8Bit(qgetenv("PATH")),
+                          kpsewhichOutput.replace(QLatin1String("\n"), QLatin1String("<br/>"))),
+                     -1);
 
         // This makes sure the we don't try to run kpsewhich again
         markFontsAsLocated();
-        delete kpsewhich_;
+        kpsewhich_.reset();
         return;
     }
     // We wait here while the external program runs concurrently.
@@ -295,19 +302,20 @@ void fontPool::locateFonts(bool makePK, bool locateTFMonly, bool *virtualFontsFo
     // Handle fatal errors.
     int const kpsewhich_exit_code = kpsewhich_->exitCode();
     if (kpsewhich_exit_code < 0) {
-        emit warning(i18n("<qt>The font generation by <em>kpsewhich</em> was aborted (exit code %1, error %2). As a "
-                          "result, some font files could not be located, and your document might be unreadable.</qt>",
-                          kpsewhich_exit_code,
-                          kpsewhich_->errorString()),
-                     -1);
+        Q_EMIT warning(i18n("<qt>The font generation by <em>kpsewhich</em> was aborted (exit code %1, error %2). As a "
+                            "result, some font files could not be located, and your document might be unreadable.</qt>",
+                            kpsewhich_exit_code,
+                            kpsewhich_->errorString()),
+                       -1);
 
         // This makes sure the we don't try to run kpsewhich again
-        if (makePK == false)
+        if (makePK == false) {
             markFontsAsLocated();
+        }
     }
 
     // Create a list with all filenames found by the kpsewhich program.
-    const QStringList fileNameList = QString::fromLocal8Bit(kpsewhich_->readAll()).split(QLatin1Char('\n'), QString::SkipEmptyParts);
+    const QStringList fileNameList = QString::fromLocal8Bit(kpsewhich_->readAll()).split(QLatin1Char('\n'), Qt::SkipEmptyParts);
 
     // Now associate the file names found with the fonts
     QList<TeXFontDefinition *>::iterator it_fontp = fontList.begin();
@@ -318,11 +326,13 @@ void fontPool::locateFonts(bool makePK, bool locateTFMonly, bool *virtualFontsFo
             QStringList matchingFiles;
 #ifdef HAVE_FREETYPE
             const QString &fn = fontsByTeXName.findFileName(fontp->fontname);
-            if (!fn.isEmpty())
+            if (!fn.isEmpty()) {
                 matchingFiles = fileNameList.filter(fn);
+            }
 #endif
-            if (matchingFiles.isEmpty() == true)
+            if (matchingFiles.isEmpty() == true) {
                 matchingFiles += fileNameList.filter(QLatin1Char('/') + fontp->fontname + QLatin1Char('.'));
+            }
 
             if (matchingFiles.isEmpty() != true) {
 #ifdef DEBUG_FONTPOOL
@@ -332,8 +342,9 @@ void fontPool::locateFonts(bool makePK, bool locateTFMonly, bool *virtualFontsFo
                 fontp->fontNameReceiver(fname);
                 fontp->flags |= TeXFontDefinition::FONT_KPSE_NAME;
                 if (fname.endsWith(QLatin1String(".vf"))) {
-                    if (virtualFontsFound != nullptr)
+                    if (virtualFontsFound != nullptr) {
                         *virtualFontsFound = true;
+                    }
                     // Constructing a virtual font will most likely insert other
                     // fonts into the fontList. After that, fontList.next() will
                     // no longer work. It is therefore safer to start over.
@@ -343,7 +354,7 @@ void fontPool::locateFonts(bool makePK, bool locateTFMonly, bool *virtualFontsFo
             }
         } // of if (fontp->filename.isEmpty() == true)
     }
-    delete kpsewhich_;
+    kpsewhich_.reset();
 }
 
 void fontPool::setCMperDVIunit(double _CMperDVI)
@@ -352,8 +363,9 @@ void fontPool::setCMperDVIunit(double _CMperDVI)
     qCDebug(OkularDviDebug) << "fontPool::setCMperDVIunit( " << _CMperDVI << " )";
 #endif
 
-    if (CMperDVIunit == _CMperDVI)
+    if (CMperDVIunit == _CMperDVI) {
         return;
+    }
 
     CMperDVIunit = _CMperDVI;
 
@@ -392,7 +404,7 @@ void fontPool::setDisplayResolution(double _displayResolution_in_dpi)
 
     // Do something that causes re-rendering of the dvi-window
     /*@@@@
-    emit fonts_have_been_loaded(this);
+    Q_EMIT fonts_have_been_loaded(this);
     */
 }
 
@@ -436,10 +448,11 @@ void fontPool::release_fonts()
 
 void fontPool::mf_output_receiver()
 {
-    const QString output_data = QString::fromLocal8Bit(kpsewhich_->readAllStandardError());
-
-    kpsewhichOutput.append(output_data);
-    MetafontOutput.append(output_data);
+    if (kpsewhich_) {
+        const QString output_data = QString::fromLocal8Bit(kpsewhich_->readAllStandardError());
+        kpsewhichOutput.append(output_data);
+        MetafontOutput.append(output_data);
+    }
 
     // We'd like to print only full lines of text.
     int numleft;
@@ -466,7 +479,7 @@ void fontPool::mf_output_receiver()
             int secondblank = startLine.lastIndexOf(QLatin1Char(' '), lastblank - 1);
             QString dpi = startLine.mid(secondblank + 1, lastblank - secondblank - 1);
 
-            emit warning(i18n("Currently generating %1 at %2 dpi...", fontName, dpi), -1);
+            Q_EMIT warning(i18n("Currently generating %1 at %2 dpi...", fontName, dpi), -1);
         }
         MetafontOutput = MetafontOutput.remove(0, numleft + 1);
     }
